orzia_map = {}
orzia_map.__index = orzia_map

function orzia_map.create(size_x, size_y)

	local map = {}          
	setmetatable(map, orzia_map)

	map.size_x = size_x
	map.size_y = size_y

	-- create empty map
	map.tiles = {}

	for x = 0, size_x - 1 do
		map.tiles[x] = {}
		for y = 0, size_y - 1 do
			map.tiles[x][y] = orzia_tile.create(true, x, y)
		end
	end

	--generate some rooms
	map.rooms = {}

	for r = 1, MAP_ROOMS_MAX do

		intersect = true

		while intersect == true do

			w = math.random(MAP_ROOMS_SIZE_MIN, MAP_ROOMS_SIZE_MAX)
			h = math.random(MAP_ROOMS_SIZE_MIN, MAP_ROOMS_SIZE_MAX)
			x = math.random(VIEW_SIZE_X, map.size_x - w - VIEW_SIZE_X)
			y = math.random(VIEW_SIZE_Y, map.size_y - h - VIEW_SIZE_Y)

			new_room = orzia_room.create(x, y, w, h)

			if r > 1 then

				intersect = false

				for c = 1, r - 1 do
					
					if new_room.x1 - 1 <= map.rooms[c].x2 and
							new_room.x2 + 1 >= map.rooms[c].x1 and
							new_room.y1 - 1 <= map.rooms[c].y2 and
							new_room.y2 + 1 >= map.rooms[c].y1 then
						intersect = true
					end
				end
			else
				intersect = false
			end

		end

		-- unblock room
		for x = new_room.x1, new_room.x2 do
			for y = new_room.y1, new_room.y2 do
				map.tiles[x][y].blocked = false
			end
		end

		-- build corridor between old room and new room
		if r > 1 then

			if math.random(0,1) == 0 then

				for x = math.min(map.rooms[r - 1].centerx, new_room.centerx), 
						math.max(map.rooms[r - 1].centerx, new_room.centerx) do
					map.tiles[x][map.rooms[r - 1].centery].blocked = false
				end

				for y = math.min(map.rooms[r - 1].centery, new_room.centery), 
						math.max(map.rooms[r - 1].centery, new_room.centery) do
					map.tiles[new_room.centerx][y].blocked = false
				end
			
			else

				for y = math.min(map.rooms[r - 1].centery, new_room.centery), 
						math.max(map.rooms[r - 1].centery, new_room.centery) do
					map.tiles[map.rooms[r - 1].centerx][y].blocked = false
				end

				for x = math.min(map.rooms[r - 1].centerx, new_room.centerx), 
						math.max(map.rooms[r - 1].centerx, new_room.centerx) do
					map.tiles[x][new_room.centery].blocked = false
				end
			end
		end

		map.rooms[r] = new_room

	end
	--end room generation

	--pave edges (set edge graphic as well)
	for x = 0, size_x - 1 do
		for y = 0, size_y - 1 do
			if x == 0 or x == map.size_x - 1 then
				map.tiles[x][y].blocked = true
				map.tiles[x][y].graphic = T_EMPTY
			elseif y == 0 or y == map.size_y - 1 then
				map.tiles[x][y].blocked = true
				map.tiles[x][y].graphic = T_EMPTY
			end
		end
	end

	-- set tile graphics(except edge)
	for x = 1, size_x - 2 do
		for y = 1, size_y - 2 do
			if map.tiles[x][y]:count_walls(map.tiles) == 9 then
				map.tiles[x][y].graphic = T_EMPTY
			elseif map.tiles[x][y].blocked == true then
				map.tiles[x][y].graphic = T_WALL[math.random(0,2)]
			elseif map.tiles[x][y].blocked == false then
				map.tiles[x][y].graphic = T_FLOOR[math.random(0,2)]
			end
		end
	end

	return map
	 
end